package	jp.co.sds_corp.syslog.filter;

import java.util.List;

import jp.co.sds_corp.syslog.SyslogEvent;

/**
* フィルタリングのツールです。
*/
public class FilterUtility
{
	private FilterUtility() {
	}

	/**
	* フィルタの一覧から、フィルタリングの結果を求めます。
	* @param	l		フィルタの格納されているList
	* @param	ev		syslog
	* @return	フィルタリングされる場合true、そうでない場合false
	*/
	public static boolean isFiltered( List<IFilter> l, SyslogEvent ev ) {
		boolean bRet = false;
		for ( int j = 0; j < l.size(); j++ ) {
			IFilter f = (IFilter )l.get( j );
			if ( f.isValid() ) {
				bRet = true;
				break;
			}
		}
		// 有効なフィルタがなければOK
		if ( !bRet ) {
			return false;
		}

		for ( int j = 0; j < l.size(); j++ ) {
			IFilter f = (IFilter )l.get( j );
			if ( !f.isValid() || !f.isExclude()) {
				continue;
			}
			// Excludeならその場で退場
			if ( f.match( ev )) {
				return true;
			}
		}
		bRet = false;
		for ( int j = 0; j < l.size(); j++ ) {
			IFilter f = (IFilter )l.get( j );
			if ( !f.isValid() || f.isExclude()) {
				continue;
			}
			bRet = true;
			// 一つでも通ればOK
			if ( f.match( ev )) {
				bRet = false;
				break;
			}
		}
		return bRet;
	}

}

