package	jp.co.sds_corp.syslog.layout;

import java.util.Iterator;
import java.util.List;

import jp.co.sds_corp.util.syslog.SyslogMsg;

/**
* 指定されたフォーマット文字列を元に、データを編集します。
*/
public abstract class PatternLayout implements ILayout
{
	private static final long serialVersionUID = 1L;
	String	fmt;
	List<IConverter>	list;
	boolean	bEscConv = true;

	/**
	* フォーマット文字列を指定します。
	* @param	fmt		フォーマット文字列
	*/
	public void setFormat( String fmt ) {
		this.fmt = fmt;
		FormatParser parser = createParser( fmt );
		parser.setEscapeConversion( bEscConv );
		list = parser.parse();
	}
	/**
	* フォーマット文字列を返します。
	* @return	フォーマット文字列
	*/
	public String getFormat() {
		return fmt;
	}

	/**
	* ESC処理を行うかどうかを設定します。
	* @param	var		ESC処理を行うかどうか
	*/
	public void setEscapeConversion( boolean var ) {
		this.bEscConv = var;
	}
	/**
	* ESC処理を行うかどうかを取得します。
	* @return	ESC処理を行うかどうか
	*/
	public boolean isEscapeConversion() {
		return this.bEscConv;
	}

	/**
	* {@link FormatParser}を生成します。
	* @param	fmt		フォーマット文字列
	*/
	public abstract FormatParser createParser( String fmt );

	/**
	* 文字列を編集して出力します。
	* @param	msg		編集元のデータ
	* @return	編集結果の文字列
	*/
	public String format( SyslogMsg msg ) {
		StringBuffer buf = new StringBuffer();
		Iterator<IConverter> itr = list.iterator();
		while ( itr.hasNext()) {
			IConverter cvt = (IConverter )itr.next();
			buf.append( cvt.convert( msg ));
		}
		return buf.toString();
	}

	/**
	* このレイアウトの文字列表現を返します。
	* @return	このレイアウトの文字列表現を返します。
	*/
	public String toString() {
		String ret = getClass().getName() + " format=[" + fmt + "]";
		return ret;
	}
}
