package jp.co.sds_corp.syslog.viewer;

import java.awt.Font;
import java.awt.Rectangle;
import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import javax.swing.Icon;

import jp.co.sds_corp.syslog.action.IAction;
import jp.co.sds_corp.syslog.action.PopupAction;
import jp.co.sds_corp.syslog.filter.IFilter;
import jp.co.sds_corp.syslog.viewer.action.IActionPanel;
import jp.co.sds_corp.syslog.viewer.filter.IFilterPanel;
import jp.co.sds_corp.util.common.CmnString;
import jp.co.sds_corp.util.config.IConfigSavable;
import jp.co.sds_corp.util.config.IConfigurable;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.config.ConfigUtility;
import jp.co.sds_corp.util.syslog.SyslogMsg;
import jp.co.sds_corp.util.window.NullIcon;
import jp.co.sds_corp.util.window.SizedIcon;

/**
* {@link SyslogViewer}の設定を保持するクラスです。
*/
public class SyslogParam implements IConfigurable, IConfigSavable
{
	class Column {
		String	name;
		String	disp;
		int		width;
		public void setName( String var ) {
			this.name = var;
			this.disp = SyslogParam.this.getDisplayName( var );
		}
		public String getName() {
			return name;
		}
		public String getDisplayName() {
			return disp;
		}
		public int getWidth() {
			return width;
		}
	}
	static class Registry {
		String type, cls, panel;
		Registry( String type, String cls, String panel ) {
			this.type = type;
			this.cls = cls;
			this.panel = panel;
		}
		public String getType() {
			return type;
		}
		public String getOrgClass() {
			return cls;
		}
		public String getPanel() {
			return panel;
		}
	}
	Registry[] actions = {
		new Registry( "File",
			"jp.co.sds_corp.syslog.action.FileAction",
			"jp.co.sds_corp.syslog.viewer.action.FilePanel" ),
		new Registry( "Log4j",
			"jp.co.sds_corp.syslog.action.Log4jAction",
			"jp.co.sds_corp.syslog.viewer.action.Log4jPanel" ),
		new Registry( "Mail",
			"jp.co.sds_corp.syslog.action.MailAction",
			"jp.co.sds_corp.syslog.viewer.action.MailPanel" ),
		new Registry( "Sound",
			"jp.co.sds_corp.syslog.action.SoundAction",
			"jp.co.sds_corp.syslog.viewer.action.SoundPanel" ),
		new Registry( "Popup",
			"jp.co.sds_corp.syslog.action.PopupAction",
			"jp.co.sds_corp.syslog.viewer.action.PopupPanel" ),
		new Registry( "Syslog",
			"jp.co.sds_corp.syslog.action.SyslogAction",
			"jp.co.sds_corp.syslog.viewer.action.SyslogPanel" )
	};
	HashMap<String,Registry> mapAction = new HashMap<String,Registry>();
	HashMap<String,Registry> mapActPnl = new HashMap<String,Registry>();
	Registry[] filters = {
		new Registry( "Level",
			"jp.co.sds_corp.syslog.filter.LevelFilter",
			"jp.co.sds_corp.syslog.viewer.filter.LevelPanel" ),
		new Registry( "Source",
			"jp.co.sds_corp.syslog.filter.SourceFilter",
			"jp.co.sds_corp.syslog.viewer.filter.SourcePanel" ),
		new Registry( "Pattern",
			"jp.co.sds_corp.syslog.filter.PatternFilter",
			"jp.co.sds_corp.syslog.viewer.filter.PatternPanel" )
	};
	HashMap<String,Registry> mapFilter = new HashMap<String,Registry>();
	HashMap<String,Registry> mapFltPnl = new HashMap<String,Registry>();

	Vector<IFilter>		vecFilter;
	Vector<String>		vecName;
	Vector<Column>		vecCol;
	String[]	defName = {
		"Icon",
		"Date",
		"Time",
		"Host",
		"Priority",
		"Facility",
		"OrgTime",
		"Tag",
		"Ident",
		"Message",
	};
	int		nWidth = 700;
	int		nHeight = 430;
	int		nX = 178;
	int		nY = 215;
	int		nRowHeight = 17;
	boolean	bAsName = false;
	boolean	bLimit = false;
	int		nMaxLogSize = 10000;
	int[]	nColOrder = null;
	URL[]	iconURL = new URL[3];
	SizedIcon[]	icon = new SizedIcon[3];
	Font	font;
	Font	defFont;
	PopupAction popup = null;

	enum Level {
		UNKNOWN, INFO, WARN, ERR
	};

	public SyslogParam() {
		vecFilter = new Vector<IFilter>();
		vecName = new Vector<String>();
		vecCol = new Vector<Column>();
		createIcon();
		for ( int i = 0; i < actions.length; i++ ) {
			mapAction.put( actions[i].cls, actions[i] );
		}
		for ( int i = 0; i < filters.length; i++ ) {
			mapFilter.put( filters[i].cls, filters[i] );
		}
	}

	protected void createIcon() {
		iconURL[0] = getClass().getResource( "/resource/info.gif" );
		iconURL[1] = getClass().getResource( "/resource/warn.gif" );
		iconURL[2] = getClass().getResource( "/resource/err.gif" );
		icon[0] = new SizedIcon( iconURL[0], 18 );
		icon[1] = new SizedIcon( iconURL[1], 18 );
		icon[2] = new SizedIcon( iconURL[2], 18 );
	}

	/**
	* 指定された初期情報で、初期化します。
	* @param	config	初期情報
	*/
	public void init( IConfiguration config ) throws Exception {
		IConfiguration c;
		String[] cols = null;
		String val;
		Iterator<IConfiguration> itr;

		val = config.getAttributes().get( "X", null );
		if ( val != null ) {
			try {
				nX = Integer.valueOf( val ).intValue();
			} catch ( NumberFormatException ex ) {}
		}
		val = config.getAttributes().get( "Y", null );
		if ( val != null ) {
			try {
				nY = Integer.valueOf( val ).intValue();
			} catch ( NumberFormatException ex ) {}
		}
		val = config.getAttributes().get( "Width", null );
		if ( val != null ) {
			try {
				nWidth = Integer.valueOf( val ).intValue();
			} catch ( NumberFormatException ex ) {}
		}
		val = config.getAttributes().get( "Height", null );
		if ( val != null ) {
			try {
				nHeight = Integer.valueOf( val ).intValue();
			} catch ( NumberFormatException ex ) {}
		}
		val = config.getAttributes().get( "RowHeight", null );
		if ( val != null ) {
			try {
				nRowHeight = Integer.valueOf( val ).intValue();
			} catch ( NumberFormatException ex ) {}
		}

		c = config.getChild( "ColumnList" );
		if ( c != null ) {
			vecCol.clear();
			vecName.clear();
			itr = c.getChilds( "Column" );
			while ( itr.hasNext() ) {
				IConfiguration c1 = (IConfiguration )itr.next();
				Column col = new Column();
				col.setName( (String )c1.getAttributes().get( "name" ));
				val = (String )c1.getAttributes().get( "width" );
				col.width = Integer.valueOf( val ).intValue();
				vecCol.add( col );
				vecName.add( col.getDisplayName() );
			}
		} else {
// @@@ 旧バージョンとの互換性
			val = config.get( "Column", null );
			if ( val != null ) {
				cols = CmnString.getStringToken( val, "," );
			}
			if ( cols == null ) {
				cols = defName;
			}
			setColumn( cols );
			val = config.get( "ColumnWidth", null );
			if ( val != null ) {
				setColWidth( val );
			}
		}
		val = config.get( "HostAsName", null );
		if ( val != null ) {
			if ( val.equals( "on" )) {
				setHostAsName( true );
			}
		}
		val = config.get( "LimitLogSize", null );
		if ( val != null ) {
			if ( val.equals( "on" )) {
				setLimitLogSize( true );
			}
		}
		val = config.get( "MaxLogSize", null );
		if ( val != null ) {
			try {
				nMaxLogSize = Integer.valueOf( val ).intValue();
			} catch ( NumberFormatException ex ) {
				nMaxLogSize = 10000;
			}
		}
		popup = new PopupAction();
		c = config.getChild( "Popup" );
		if ( c != null ) {
			popup.init( c );
		} else {
			popup.setValid( false );
		}
		c = config.getChild( "Font" );
		if ( c != null ) {
			String name = c.getAttributes().get( "name" );
			int nStyle;
			String style = c.getAttributes().get( "style" );
			if ( style.equals( "ITALIC" )) {
				nStyle = Font.ITALIC;
			} else if ( style.equals( "BOLD" )) {
				nStyle = Font.BOLD;
			} else if ( style.equals( "BOLD+ITALIC" )) {
				nStyle = Font.BOLD + Font.ITALIC;
			} else {
				nStyle = Font.PLAIN;
			}
			int nSize = Integer.valueOf( c.getAttributes().get( "size" ))
							.intValue();
			font = new Font( name, nStyle, nSize );
		}

		itr = config.getChilds( "Filter" );
		while ( itr.hasNext() ) {
			c = (IConfiguration )itr.next();
			IFilter f = (IFilter )ConfigUtility.createInstance( c );
			f.init( c );
			vecFilter.add( f );
		}
		itr = config.getChilds( "ActionDefine" );
		while ( itr.hasNext() ) {
			c = (IConfiguration )itr.next();
			Registry reg = new Registry(
				c.getAttributes().get( "type" ),
				c.getAttributes().get( "action" ),
				c.getAttributes().get( "panel" ) );
			mapAction.put( reg.cls, reg );
		}
		itr = config.getChilds( "FilterDefine" );
		while ( itr.hasNext() ) {
			c = (IConfiguration )itr.next();
			Registry reg = new Registry(
				c.getAttributes().get( "type" ),
				c.getAttributes().get( "filter" ),
				c.getAttributes().get( "panel" ) );
			mapFilter.put( reg.cls, reg );
		}
	}

	private void getDefault() {
		for ( int i = 0; i < defName.length; i++ ) {
			Column col = new Column();
			col.setName( defName[i] );
			vecCol.add( col );
			vecName.add( col.getDisplayName() );
		}
	}

	protected String getDisplayName( String name ) {
		if ( name.equals( "Icon" )) {
			return " ";
		}
		return name;
	}

	public int getColumnCount() {
		return vecCol.size();
	}

	public String getColumnName( int nCol ) {
		return (String )vecName.get( nCol );
	}

	public int getColumnPos( String name ) {
		return vecName.indexOf( getDisplayName( name ));
	}

	public void setColumn( java.util.List<String> cols ) {
		vecCol.clear();
		vecName.clear();
		for ( int i = 0; i < cols.size(); i++ ) {
			Column col = new Column();
			col.setName( (String )cols.get( i ));
			vecCol.add( col );
			vecName.add( col.getDisplayName() );
		}
	}

	public void setColumn( String[] cols ) {
		vecCol.clear();
		vecName.clear();
		for ( int i = 0; i < cols.length; i++ ) {
			Column col = new Column();
			col.setName( cols[i] );
			vecCol.add( col );
			vecName.add( col.getDisplayName() );
		}
	}

	public java.util.List<String> getAllColumn() {
		Vector<String> ret = new Vector<String>();
		for ( int i = 0; i < defName.length; i++ ) {
			ret.add( defName[i] );
		}
		return ret;
	}
	public java.util.List<String> getSelectedColumn() {
		Vector<String> ret = new Vector<String>();
		for ( int i = 0; i < vecCol.size(); i++ ) {
			Column col = (Column )vecCol.get( i );
			ret.add( col.getName() );
		}
		return ret;
	}

	public Level getLevel( SyslogMsg msg ) {
		Level level = Level.UNKNOWN;
		switch ( msg.getPriority().getValue() ) {
		case 0: case 1: case 2: case 3:
			level = Level.ERR;
			break;
		case 4:
			level = Level.WARN;
			break;
		case 5: case 6: case 7:
		default:
			level = Level.INFO;
			break;
		}
		return level;
	}

	public Icon getIcon( SyslogMsg msg ) {
		return getIcon( getLevel( msg ));
	}
	public Icon getIcon( Level level ) {
		Icon i = null;
		switch ( level ) {
		case UNKNOWN: i = new NullIcon(); break;
		case ERR:  i = icon[0]; break;
		case WARN: i = icon[1]; break;
		case INFO: i = icon[2]; break;
		}
		return i;
	}

	public Icon getIcon( SyslogMsg msg, int size ) {
		return getIcon( getLevel( msg ), size );
	}
	public Icon getIcon( Level level, int size ) {
		Icon i = null;
		if ( level == Level.UNKNOWN ) {
			i = new NullIcon();
		} else {
			URL url = null;
			switch ( level ) {
			case INFO:  url = iconURL[0]; break;
			case WARN: url = iconURL[1]; break;
			case ERR: url = iconURL[2]; break;
			default:
				break;
			}
			i = new SizedIcon( url, size );
		}
		return i;
	}

	/**
	* 設定されている内容を出力します。
	* @param    config  設定情報
	*/
	public void saveConfig( IConfiguration config ) {
		IConfiguration c;
		for ( int i = 0; i < vecFilter.size(); i++ ) {
			IFilter filter = (IFilter )vecFilter.get( i );
			c = config.addChild( "Filter" );
			filter.saveConfig( c );
		}
		config.setAttribute( "Width", String.valueOf( nWidth ));
		config.setAttribute( "Height", String.valueOf( nHeight ));
		config.setAttribute( "X", String.valueOf( nX ));
		config.setAttribute( "Y", String.valueOf( nY ));
		config.setAttribute( "RowHeight", String.valueOf( nRowHeight ));
		config.addChild( "HostAsName", getHostAsName() ? "on" : "off" );
		config.addChild( "LimitLogSize", getLimitLogSize() ? "on" : "off" );
		config.addChild( "MaxLogSize", String.valueOf( nMaxLogSize ));
		if ( popup != null ) {
			c = config.addChild( "Popup" );
			popup.saveConfig( c );
		}
		if ( font != null ) {
			c = config.addChild( "Font" );
			c.setAttribute( "name", font.getName() );
			c.setAttribute( "size", String.valueOf( font.getSize()) );
			String s;
			if ( font.isItalic()) {
				if ( font.isBold()) {
					s = "BOLD+ITALIC";
				} else {
					s = "ITALIC";
				}
			} else {
				if ( font.isBold()) {
					s = "BOLD";
				} else {
					s = "PLAIN";
				}
			}
			c.setAttribute( "style", s );
		}

		java.util.List<String> col = getSelectedColumn();
		int[] cw = getColumnWidth();
		IConfiguration list = config.addChild( "ColumnList" );
		for ( int i = 0; i < col.size(); i++ ) {
			c = list.addChild( "Column" );
			c.setAttribute( "name", (String )col.get( nColOrder[i] ));
			c.setAttribute( "width", String.valueOf( cw[i] ));
		}
	}

	public void setWidth( int nWidth ) {
		this.nWidth = nWidth;
	}
	public int getWidth() {
		return nWidth;
	}

	public void setHeight( int nHeight ) {
		this.nHeight = nHeight;
	}
	public int getHeight() {
		return nHeight;
	}

	public void setX( int nX ) {
		this.nX = nX;
	}
	public int getX() {
		return nX;
	}

	public void setY( int nY ) {
		this.nY = nY;
	}
	public int getY() {
		return nY;
	}

	public Rectangle getBounds() {
		Rectangle ret = new Rectangle( nX, nY, nWidth, nHeight );
		return ret;
	}

	public void setBounds( Rectangle rect ) {
		nX = (int )rect.getX();
		nY = (int )rect.getY();
		nWidth = (int )rect.getWidth();
		nHeight = (int )rect.getHeight();
	}

	public void setColWidth( String val ) {
		String[] buf = CmnString.getStringToken( val, "," );
		for ( int i = 0; i < vecCol.size(); i++ ) {
			Column col = (Column )vecCol.get( i );
			try {
				col.width = Integer.valueOf( buf[i] ).intValue();
			} catch ( NumberFormatException ex ) {
				return;
			}
		}
	}
	public void setColWidth( int[] nColWidth ) {
		for ( int i = 0; i < nColWidth.length; i++ ) {
			Column col = (Column )vecCol.get( i );
			col.width = nColWidth[i];
		}
	}
	public int[] getColumnWidth() {
		int[] ret = new int[vecCol.size()];
		for ( int i = 0; i < vecCol.size(); i++ ) {
			Column col = (Column )vecCol.get( i );
			ret[i] = col.width;
		}
		return ret;
	}

	public void setOrder( int[] nOrder ) {
		this.nColOrder = nOrder;
	}

	public boolean getHostAsName() {
		return bAsName;
	}
	public void setHostAsName( boolean var ) {
		this.bAsName = var;
	}

	public boolean getLimitLogSize() {
		return bLimit;
	}
	public void setLimitLogSize( boolean var ) {
		this.bLimit = var;
	}

	public int getMaxLogSize() {
		return nMaxLogSize;
	}
	public void setMaxLogSize( int var ) {
		this.nMaxLogSize = var;
	}

	public PopupAction getPopup() {
		return popup;
	}

	public Font getDefaultFont() {
		if ( defFont == null ) {
			defFont = new Font( "Dialog", Font.PLAIN, 14 );
		}
		return defFont;
	}
	public Font getFont() {
		if ( font == null ) {
			font = getDefaultFont();
		}
		return font;
	}
	public void setFont( Font font ) {
		this.font = font;
	}

	public java.util.List<IFilter> listFilters() {
		return vecFilter;
	}

	public void setRowHeight( int var ) {
		this.nRowHeight = var;
	}
	public int getRowHeight() {
		return nRowHeight;
	}

	/**
	* 登録されている{@link IFilter}の一覧を取得します。
	*/
	public Map<String,Registry> getFilterMap() {
		return mapFilter;
	}
	/**
	* {@link IFilter}と対応する{@link IFilterPanel}のタイプ名を取得します。
	*/
	public Registry getFilterType( IFilter filter ) {
		return (Registry )mapFilter.get( filter.getClass().getName() );
	}
	/**
	* {@link IFilter}と対応する{@link IFilterPanel}を取得します。
	*/
	public IFilterPanel getFilterPanel( IFilter filter ) throws Exception {
		String cls = filter.getClass().getName();
		IFilterPanel ret = (IFilterPanel )mapFltPnl.get( cls );
		if ( ret == null ) {
			Registry reg = (Registry )mapFilter.get( cls );
			if ( reg == null ) {
				return null;
			}
			String pnlCls = reg.panel;
			ret = (IFilterPanel )Class.forName( pnlCls ).newInstance();
		}
		return ret;
	}

	/**
	* 登録されている{@link IAction}の一覧を取得します。
	*/
	public Map<String,Registry> getActionMap() {
		return mapAction;
	}
	/**
	* {@link IAction}と対応する{@link IActionPanel}のタイプ名を取得します。
	*/
	public Registry getActionType( IAction action ) {
		return (Registry )mapAction.get( action.getClass().getName() );
	}
	/**
	* {@link IAction}と対応する{@link IActionPanel}を取得します。
	*/
	public IActionPanel getActionPanel( IAction action ) throws Exception {
		String cls = action.getClass().getName();
		IActionPanel ret = (IActionPanel )mapActPnl.get( cls );
		if ( ret == null ) {
			Registry reg = (Registry )mapAction.get( cls );
			if ( reg == null ) {
				return null;
			}
			String pnlCls = reg.panel;
			ret = (IActionPanel )Class.forName( pnlCls ).newInstance();
		}
		return ret;
	}

}

